from rapidfuzz import fuzz, process

def custom_ratio(s1, s2, score_cutoff=0):
    # 공백을 제거한 문자열로 비교하되, 원본 문자열은 변경하지 않음
    return fuzz.ratio(s1.replace(" ", ""), s2.replace(" ", ""), score_cutoff=score_cutoff)

def group_products(df, it_threshold=96, etc_threshold=95):
    # IT / ETC 제품간 임계값 설정
    product_names = df['TITLE'].tolist()
    grouped_names = []
    scores = []

    for index, row in df.iterrows():
        threshold = it_threshold if row['Type'] == 'IT' else etc_threshold

        product_name = row['TITLE']
        best_match = None
        best_score = 0

        for grouped_name in grouped_names:
            score = custom_ratio(product_name, grouped_name)
            if score >= threshold and score > best_score:
                best_match = grouped_name
                best_score = score

        if best_match:
            grouped_names.append(best_match)
            scores.append(best_score)
        else:
            grouped_names.append(product_name)
            scores.append(100)

    df['Grouped Name'] = grouped_names
    df['Score'] = scores
    return df

def update_grouped_titles(df):
    # Get unique group names
    unique_group_names = df['Grouped Name'].unique()

    # Create a dictionary to store representative names for each group
    representative_names = {}

    for group_name in unique_group_names:
        # Get rows belonging to the current group
        group = df[df['Grouped Name'] == group_name]

        # Count the frequency of each title within the group
        title_counts = group['TITLE'].value_counts()

        # Select the most frequent title as the representative name
        representative_name = title_counts.index[0]

        # Store the representative name for the current group
        representative_names[group_name] = representative_name

    # Update the 'Grouped Name' column with representative names
    df['Grouped Name'] = df['Grouped Name'].map(representative_names)

    # Calculate the score between original title and representative name
    df['Score'] = df.apply(lambda row: custom_ratio(row['TITLE'], row['Grouped Name']), axis=1)

    # Check if the title has been modified
    df['Modified'] = df['TITLE'] != df['Grouped Name']

    return df



def handle_process_text_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AttributeError:
            print("MeCab object 'mecab' is not defined or doesn't have 'morphs' method")
            return "", ""
        except Exception as e:
            print(f"An error occurred in process_text: {str(e)}")
            return "", ""
    return wrapper

def handle_find_storage_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print(f"An error occurred in find_storage: {str(e)}")
            return 'N/A', 'N/A'
    return wrapper


from konlpy.tag import Mecab
import pandas as pd
import filter_list
import cleaning
import handler

# Attribute call Func
def get_product_type(attributes, product_type_dict):
    def classify_type(attribute):
        if attribute in product_type_dict:
            return product_type_dict[attribute]
        else:
            return 'ETC'

    return attributes.map(classify_type)


# ETC 제목 추출 함수
def title_ETC(description):
    name = description
    return name


# IT 제목 추출 함수
def process_text(morphs, title_list):
    if not isinstance(morphs, list):
        raise TypeError("Input 'morphs' must be a list")
    if not isinstance(title_list, list):
        raise TypeError("Input 'title_list' must be a list")

    title = []
    i = 0
    plus_used = False
    while i < len(morphs):
        if morphs[i] in title_list:
            title.append(morphs[i])
            i += 1
            if i < len(morphs) and morphs[i].isdigit():
                title.append(morphs[i])
                i += 1
            if i < len(morphs) and morphs[i] == '+' and not plus_used:
                title.append(morphs[i])
                plus_used = True
                i += 1
        else:
            i += 1

    return ''.join(title).strip()

def create_dataframe_advanced(df, title, product_type_dict, stopwords, color_list, attribute):
    filtered_text = advanced_cleaning(title, stopwords, color_list)  # Data Cleaning

    extracted_title = []
    extracted_color = []
    extracted_storage = []
    mecab = Mecab()

    # Get product types for all items at once
    product_types = get_product_type(attribute, product_type_dict)

    for text, product_type in zip(filtered_text, product_types):
        if product_type == 'IT':
            mecabmorphs = mecab.morphs(text['cleaned_text'])  # 형태소 추출
            # 제목 형태소 추출
            extracted_title_temp = process_text(mecabmorphs, product_title)
            if len(extracted_title_temp.split()) <= 2:  # 추출된 제목이 2단어 이하인 경우
                extracted_title.append(text['cleaned_text'])  # 정제된 텍스트 사용
            else:
                extracted_title.append(extracted_title_temp)
            extracted_color.append(text['color'])
            extracted_storage.append(text['storage'])
        else:
            extracted_title.append(title_ETC(text['cleaned_text']))
            extracted_color.append(text['color'])
            extracted_storage.append('N/A')

    # 제목에서 색상과 용량 정보 제거
    for i in range(len(extracted_title)):
        if extracted_color[i] != 'N/A':
            extracted_title[i] = extracted_title[i].replace(extracted_color[i], '').strip()
        if extracted_storage[i] != 'N/A':
            extracted_title[i] = extracted_title[i].replace(extracted_storage[i], '').strip()

    # Convert lists to Series
    extracted_title = pd.Series(extracted_title)
    extracted_color = pd.Series(extracted_color)
    extracted_storage = pd.Series(extracted_storage)
    extracted_type = pd.Series(product_types)
    original_title = pd.Series(df["DESCRIPTION"])
    dfattributes = pd.Series(attribute)

    cleaned_df = pd.DataFrame({
        'COLOR': extracted_color,
        'STORAGE': extracted_storage,
        'Type': extracted_type,
        'Classification' : dfattributes,
        'Original': original_title,
        'TITLE': extracted_title
        })

    return cleaned_df


import re

def advanced_cleaning(descriptions, stopwords, color_list):
    if isinstance(descriptions, str):
        descriptions = [descriptions]

    stopwords_set = set(stopwords)
    stopwords_pattern = re.compile(r'\b(' + '|'.join(re.escape(sw) for sw in stopwords_set) + r')\b', re.IGNORECASE)

    cleaned_descriptions = []

    for description in descriptions:
        if '점]' in description:
            description = description.split('점]')[-1]

        # 모든 알파벳을 대문자로 변환
        description = description.upper()

        # 괄호 내용 처리
        description, bracket_storage, bracket_color = process_brackets(description, color_list)

        # 일반 텍스트에서 용량 및 색상 추출
        general_storage = find_storage1(description)
        general_color = color_extraction1(description, color_list)

        # 최종 용량 및 색상 결정
        storage = bracket_storage if bracket_storage != 'N/A' else general_storage
        color = bracket_color if bracket_color != 'N/A' else general_color

        # 불용어 및 특수 문자 제거
        description = re.sub(r'[^A-Z0-9ㄱ-ㅎㅏ-ㅣ가-힣+\s²-]', ' ', description)
        description = stopwords_pattern.sub('', description)

        # 추가 정제: 연속된 공백 제거
        description = re.sub(r'\s+', ' ', description).strip()

        cleaned_descriptions.append({
            'cleaned_text': description,
            'storage': storage,
            'color': color
        })

    return cleaned_descriptions if len(cleaned_descriptions) > 1 else cleaned_descriptions[0]

def process_brackets(text, color_list):
    storage = 'N/A'
    color = 'N/A'

    def extract_from_bracket(match):
        nonlocal storage, color
        content = match.group(0)  # 괄호 내용 전체 캡처
        bracket_storage = find_storage1(content)
        bracket_color = color_extraction1(content, color_list)
        if bracket_storage != 'N/A':
            storage = bracket_storage
        if bracket_color != 'N/A':
            color = bracket_color
        return ''  # 괄호 내용과 괄호 기호 제거

    # 괄호 내용 처리
    bracket_pattern = r'[\(\[\{][^\)\]\}]+[\)\]\}]'
    text = re.sub(bracket_pattern, extract_from_bracket, text)

    return text, storage, color

def find_storage1(text):
    if not isinstance(text, str):
        raise TypeError("Input 'text' must be a string")
    storage_patterns = [
        r'(?:\d+\s*[GT]B\s*\+\s*)*(\d+)\s*([GT]B)',
        r'(\d+)\s*([GT]B)',
        r'(\d+)\s*(G)'
    ]
    for pattern in storage_patterns:
        match = re.search(pattern, text, re.IGNORECASE)
        if match:
            storage_value = int(match.group(1))
            storage_unit = match.group(2).upper()
            if storage_unit == 'TB':
                storage_value *= 1024
            if storage_value < 64:
                return 'N/A'
            else:
                return f'{storage_value}GB'
    return 'N/A'

def color_extraction1(text, color_list):
    color_pattern = r'\b(' + '|'.join(re.escape(color) for color in color_list) + r')\b'
    match = re.search(color_pattern, text, re.IGNORECASE)
    if match:
        return match.group(1)
    else:
        return 'N/A'



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="HMautogroup",
    version="0.3.2",
    author="KDPark",
    author_email="k602511@gmail.com",
    description="auto-cleaning-grouping-process",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kdpark0284/HM_titledescription_filtering",
    project_urls={
        "Bug Tracker": "https://github.com/kdpark0284/HM_titledescription_filtering/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "openpyxl", 
        "numpy", 
        "pandas", 
        "regex",  
        "konlpy", 
        "setuptools",
        "mecab-python",
        "mecab-python3",
        "rapidfuzz",
        "jamo",
        "wheel"
    ],
    entry_points={
        'console_scripts': [
            'hmautogroup=HMautogroup.main:main',  # 'main:main'은 main.py 파일의 main 함수를 의미합니다.
        ],
    },
    package_dir={"HMautogroup": "lib/HMautogroup"},
    packages=["HMautogroup"],
    package_data={'': ['LICENSE.txt', 'requirements.txt']},
    include_package_data=True,
    python_requires=">=3.10.12",
)

# python setup.py bdist_wheel
# twine upload dist/ypcc-0.2-py3-none-any.whl

# python setup.py sdist bdist_wheel
# twine upload dist/*


from text_processing import create_dataframe_advanced
from similarity import group_products, update_grouped_titles

def main(df, title, product_type_dict, stopwords, color_list, attributes):
    Filtered_DF = create_dataframe_advanced(df, title, product_type_dict, stopwords, color_list, attributes)
    grouped_df = group_products(Filtered_DF)
    updated_df = update_grouped_titles(grouped_df)
    return updated_df
